"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
;
const DEFAULT_AUTOMATA_STATE_KEY = 'state';
function createAutomataReducer(automata, init) {
    const key = arguments[2] || DEFAULT_AUTOMATA_STATE_KEY;
    if (!isString(key)) {
        return createAutomataReducer(automata, init, void 0, key);
    }
    const toStandardAction = arguments[3] || identity;
    const _automata = preprocess(automata, key);
    return function (previous = { [key]: init }, event) {
        const { type } = toStandardAction(event);
        const reducers = _automata[previous[key]][type] || [];
        let i = reducers.length;
        let state = previous;
        while (i--) {
            state = reducers[i](state, event);
        }
        return state;
    };
}
exports.default = createAutomataReducer;
function preprocess(automata, key) {
    return Object.keys(automata).reduce(function (states, state) {
        const events = automata[state];
        states[state] = Object.keys(events).reduce(function (reducers, type) {
            reducers[type] = [].concat(events[type]).map(withAutomataReducer);
            return reducers;
        }, {});
        return states;
    }, {});
    function withAutomataReducer(reducer) {
        return !isString(reducer)
            ? reducer
            : function (state) {
                return state && state[key] === reducer
                    ? state
                    : Object.assign({}, state, { [key]: reducer });
            };
    }
}
function isString(v) {
    return typeof (v && v.valueOf()) === 'string';
}
function identity(v) {
    return v;
}
